@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2022-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2022-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -54,8 +54,7 @@
 %%--------------------------------------------------------------------
 %% Common Test interface functions -----------------------------------
 %%--------------------------------------------------------------------
-suite() -> [{ct_hooks,[ts_install_cth]},
-            {timetrap,{seconds,60}}].
+suite() -> [{timetrap,{seconds,60}}].
 
 all() -> [tc_basic, tc_no_trace, tc_api_profile, tc_rle_profile,
           tc_budget_option, tc_write, tc_file_option, tc_check_profiles].
@@ -140,16 +139,16 @@ tc_api_profile(Config) ->
         #{
           call =>
               [{"    (server) -> ssl:handshake/2", ssl, handshake},
-               {"    (server) -> ssl_gen_statem:initial_hello/3",
-                ssl_gen_statem, initial_hello},
-               {"    (client) -> ssl_gen_statem:initial_hello/3",
-                ssl_gen_statem, initial_hello}],
+               {"    (server) -> tls_server_connection:initial_hello/3",
+                tls_server_connection, initial_hello},
+               {"    (client) -> tls_client_connection:initial_hello/3",
+                tls_client_connection, initial_hello}],
          return_from =>
               [{"    (server) <- ssl:listen/2 returned", ssl, listen},
-               {"    (server) <- ssl_gen_statem:initial_hello/3 returned",
-                ssl_gen_statem, initial_hello},
-               {"    (client) <- ssl_gen_statem:initial_hello/3 returned",
-                ssl_gen_statem, initial_hello},
+               {"    (server) <- tls_server_connection:initial_hello/3 returned",
+                tls_server_connection, initial_hello},
+               {"    (client) <- tls_client_connection:initial_hello/3 returned",
+                tls_client_connection, initial_hello},
                {"    (client) <- ssl_gen_statem:connect/8 returned",
                 ssl_gen_statem, connect},
                {"    (client) <- ssl:connect/3 returned", ssl, connect},
@@ -204,9 +203,12 @@ tc_api_profile(Config) ->
     check_trace_map(Ref, TracesAfterConnect, UnhandledTraceCnt1),
     ssl_test_lib:close(Server),
     ssl_test_lib:close(Client),
+    %% terminate_alert will get called twice by both client and
+    %% server to strip away Details from {shutdown::Reason, Detatils}
+    %% before matching the Reason
     UnhandledTraceCnt2 =
-        #{call => 0, processed => no_trace_received, exception_from => 0,
-          return_from => 0},
+        #{call => 2, processed => no_trace_received, exception_from => 0,
+          return_from => 2},
     check_trace_map(Ref, TracesAfterDisconnect, UnhandledTraceCnt2),
     ssl_trace:stop(),
     ok.
